@@ -41,11 +41,14 @@ def parse_subject(
 
     if len(tokens) < minlen:
         raise InvalidSubject(
-            "subject must contain %s tokens at least, got %s" % (minlen, subject)
+            "subject must contain %s tokens at least, got %s"
+            % (minlen, subject)
         )
 
     if tokens[: len(package_subject)] != package_subject:
-        raise InvalidSubject("subject should start with %s" % ".".join(package_subject))
+        raise InvalidSubject(
+            "subject should start with %s" % ".".join(package_subject)
+        )
 
     tokens = tokens[len(package_subject) :]
 
@@ -54,7 +57,8 @@ def parse_subject(
 
     if tokens[: len(service_subject)] != service_subject:
         raise InvalidSubject(
-            "subject should contain %s, got %s" % (".".join(service_subject), subject)
+            "subject should contain %s, got %s"
+            % (".".join(service_subject), subject)
         )
 
     tokens = tokens[len(service_subject) :]
@@ -106,13 +110,11 @@ async def streamed_reply_request(nc, subject, req, timeout):
     heartbeat_task = asyncio.ensure_future(heartbeat())
 
     sid = await nc.subscribe(inbox, cb=handler)
-
     try:
         await nc.publish_request(subject, inbox, req)
 
         while True:
             msg = await asyncio.wait_for(msg_queue.get(), timeout)
-
             # Is it a keep-alive msg ?
             if len(msg.data) == 1 and msg.data[0] == 0:
                 continue
@@ -122,28 +124,32 @@ async def streamed_reply_request(nc, subject, req, timeout):
                 err = nrpc_pb2.Error.FromString(msg.data[1:])
                 if err.type == nrpc_pb2.Error.EOS:
                     return
-                raise Error
+                raise err
 
             yield msg
     finally:
         heartbeat_task.cancel()
         await nc.unsubscribe(sid)
+        try:
+            await heartbeat_task
+        except asyncio.CancelledError:
+            pass
 
 
 async def heartbeat_listener(nc, subject, oncancel):
     queue = asyncio.Queue(1)
 
-    def handler(msg):
-        queue.put(msg)
+    async def handler(msg):
+        await queue.put(msg)
 
-    sid = await nc.subscribe(nc, subject, handler)
+    sid = await nc.subscribe(subject, cb=handler)
 
     try:
         while True:
             try:
                 msg = await asyncio.wait_for(queue.get(), 2)
                 beat = nrpc_pb2.HeartBeat.FromString(msg.data)
-                if beat.last:
+                if beat.lastbeat:
                     oncancel()
                     return
             except asyncio.TimeoutError as e:
@@ -225,3 +231,24 @@ async def streamed_reply_handler(nc, inbox, async_gen):
     finally:
         task.cancel()
         heartbeat_task.cancel()
+        failures = []
+        try:
+            await task
+        except asyncio.CancelledError:
+            pass
+        except Exception as e:
+            failures.append(e)
+        try:
+            await heartbeat_task
+        except asyncio.CancelledError:
+            pass
+        except Exception as e:
+            failures.append(e)
+
+        if len(failures) == 1:
+            raise failures[0]
+        if len(failures) != 0:
+            raise RuntimeError(
+                "caught multiple errors: "
+                + ", ".join(str(e) for e in failures)
+            )
